package biomorphs;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class BiomorphPanel extends JPanel {

    private Genome biomorph;
    private Color color = java.awt.Color.BLACK;
    private int dx[];
    private int dy[];
    /** setted to true if this component was clicked recently */
    private boolean clicked = false;
    
    public void setGenome(Genome biomorph) {
        this.biomorph = biomorph;
        dx = new int[biomorph.getNoOfDirections()];
        dy = new int[biomorph.getNoOfDirections()];
        this.decodeDirectionGenes();
    }
    
    public Genome getGenome() {
        return biomorph;
    }

    private void decodeDirectionGenes() {
        // basic x directions
        dx[3] = -biomorph.getGeneAt(0);
        dx[4] = -biomorph.getGeneAt(1);
        dx[5] = -biomorph.getGeneAt(2);
        // symetry about y axis
        dx[1] = -dx[3];
        dx[0] = -dx[4];
        dx[7] = -dx[5];
        // directions 2 and 6 are along y
        dx[2] = 0;
        dx[6] = 0;
        
        // basic y directions
        dy[2] = -biomorph.getGeneAt(3);
        dy[3] = -biomorph.getGeneAt(4);
        dy[4] = -biomorph.getGeneAt(5);
        dy[5] = -biomorph.getGeneAt(6);
        dy[6] = -biomorph.getGeneAt(7);
        // symetry about x axis
        dy[0] = dy[4];
        dy[1] = dy[3];
        dy[7] = dy[5];
    }
    
    public BiomorphPanel(Genome biomorph) {
        this.setGenome(biomorph);
    }
    
    public BiomorphPanel(Genome biomorph, Color color) {
        this.setGenome(biomorph);
        this.color = color;
    }
    
    public void paintComponent(Graphics g) {
        super. paintComponent(g);
        
        g.setColor( Color.black );
        
        Insets border = getInsets();
        int width = getWidth() - border.left - border.right;
        int height = getHeight() - border.top - border.bottom;
        int x = ( width / 2 ) + border.left;
        int y = ( height / 2 ) + 10 + border.top;
        
        draw(g, x, y, biomorph.getGeneAt(8), 2);
        
    }
    
    private void draw(Graphics g, int x, int y, int length, int direction) {
        direction = (direction+ 8)%8;
        
        g.setColor(color);
        
        float scale = (float)java.lang.Math.ceil(biomorph.getGeneAt(8)/3);
        if (scale<1) scale = 1;
        scale += 0.5;
        int newX = x + (int)(length * dx[direction]/scale);
        int newY = y - (int)(length * dy[direction]/scale);
        g.drawLine(x, y, newX, newY);
        if ( length > 0 ) {
            draw(g, newX, newY, length-1, direction-1);
            draw(g, newX, newY, length-1, direction+1);
        }
    }
    
    public boolean getClicked() {
        return clicked;
    }
    
    public void setClicked(boolean clicked) {
        this.clicked = clicked;
    }
    
    MouseListener ml = new MouseListener() {
        public void mouseClicked(MouseEvent e) {
            setClicked(true);
        }
        public void mouseEntered(MouseEvent e) {
        }
        public void mouseExited(MouseEvent e) {
        }
        public void mousePressed(MouseEvent e) {
        }
        public void mouseReleased(MouseEvent e) {
        }
    };
    
    
}