package biomorphs;

import java.util.Random;

public class Genome implements java.lang.Cloneable {
    /** Number of genes
     * 8 genes code direction
     * 1 gene code length
     */
    private static int noOfGenes = 9;
    
    /** Number of directions */
    private static int noOfDirections = 8;
    
    /** Place where value of each gene is stored */
    private int genes[] = new int[noOfGenes];
    
    public Genome() {

        /*
        // initialoze default values for genome
        // works exactly opposite way than in Figure 3 in Dawkins book
        genes[0] = 5;
        genes[1] = 5;
        genes[2] = 5;
        genes[3] = 5;
        genes[4] = 5;
        genes[5] = 0;
        genes[6] = -5;
        genes[7] = -5;
        */
        
        // initialoze default values for original genome
        genes[0] = -5;
        genes[1] = -5;
        genes[2] = -5;
        genes[3] = -5;
        genes[4] = -5;
        genes[5] = 0;
        genes[6] = 5;
        genes[7] = 5;
        
        // set default number of lenght of biomorph
        genes[8] = 6;
    }
    
    public void randomizeGenome() {
        genes[0] = randomNoBetween(-9,9);
        genes[1] = randomNoBetween(-9,9);
        genes[2] = randomNoBetween(-9,9);
        genes[3] = randomNoBetween(-9,9);
        genes[4] = randomNoBetween(-9,9);
        genes[5] = randomNoBetween(-9,9);
        genes[6] = randomNoBetween(-9,9);
        genes[7] = randomNoBetween(-9,9);
        genes[8] = randomNoBetween(2,9);
    }
    
    private int randomNoBetween(int a, int b) {
        int scale = b-a;
        
        Random generator = new Random();
        int randomIndex = generator.nextInt(scale+1) + a; // numbers between a and b
        return randomIndex;
    }
    
    /** Return how many genes are used to code directions. */
    public int getNoOfDirections() {
        return noOfDirections;
    }
    
    /** Return count of genes. */
    public int getNoOfGenes() {
        return noOfGenes;
    }
    
    /** Get value of gene. */
    public int getGeneAt(int index) {
        if (index < noOfGenes && index >= 0) {
            return genes[index];
        }
        else throw new IllegalStateException();
    }
    
    /** Change value of gene.
     * It will be changed only if the following constraints are satisfied.
     * Genes: 1-8 only 19 values are allowed from -9 to 9 including
     * Gene: 9 (index 8) only values from 0-9 including
     */
    public void setGeneAt(int index, int value) {
        if (index <= 7 && value <= 9 && value >= -9) {
            genes[index] = value;
        }
        if (index == 8 && value <= 9 && value >= 0) {
            genes[index] = value;
        }
    }
    
    /**
     * Increase value of gene.
     * Again constraints are checked. see method setGeneAt
     */
    public void increaseGene(int index) {
        if (index <= 7 && genes[index] <= 8) {
            genes[index]++;
        }
        if (index == 8 && genes[index] <= 8) {
            genes[index]++;
        }
    }
    
    /**
     * Decrease value of gene.
     * Again constraints are checked. see method setGeneAt
     */
    public void decreaseGene(int index) {
        if (index <= 7 && genes[index] >= -8) {
            genes[index]--;
        }
        if (index == 8 && genes[index] >= 1) {
            genes[index]--;
        }
    }
    
    public Genome clone() throws CloneNotSupportedException {
        Genome clone = (Genome)super.clone();
        
        int newGenes[] = genes.clone();
        clone.genes = newGenes;
        
        return clone;
        //return (Genome)super.clone();
    }

    public String toString() {
        StringBuilder s = new StringBuilder();
        for (int i=0; i < noOfGenes; i++) {
            
            s.append(genes[i]);
            if (i!=noOfGenes-1) {
                s.append(" ");
            }
        }
        return s.toString();
    }
    
}
